@@ -6,6 +6,7 @@ from django import template
 from django.shortcuts import render_to_response, redirect
 
 import models
+import utils
 
 
 class ConfigAdmin(admin.ModelAdmin):
@@ -13,33 +14,41 @@ class ConfigAdmin(admin.ModelAdmin):
     def has_add_permission(self, request):
         return False
     
+    def has_change_permission(self, request, instance=None):
+        return instance is None
+    
+    has_delete_permission = has_change_permission
+    
     @csrf_protect_m
     def changelist_view(self, request, extra_context=None):
         valid = True
         forms = []
-        for name, cls in models.registry:
-            form_cls = cls.admin_form()
+        for name, group_cls in utils.registry:
+            form_class = group_cls.Meta.form_class
+            form_initial = group_cls.Meta.form_initial
             if request.method == "POST":
-                form = form_cls(request.POST)
+                form = form_class(request.POST, request.FILES,
+                                  initial=form_initial)
                 if not form.is_valid():
                     valid = False
             else:
-                form = form_cls()
+                form = form_class(initial=form_initial)
+            title = getattr(group_cls, "verbose_name", None)
+            if not isinstance(title, basestring):
+                title = group_cls.__name__
             forms.append({
-                "cls": cls,
-                "name": cls.verbose_name,
-                "form": form
+                "cls":  group_cls,
+                "name": title,
+                "form": form,
             })
         if request.method == "POST" and valid:
             for form in forms:
-                for name, value in form["form"].cleaned_data.items():
-                    name = name.split(".")[1]
-                    setattr(form["cls"].config, name, value)
-            self.message_user(request, u"Config values saved")
+                form["cls"].Meta.update(**form["form"].cleaned_data)
+            self.message_user(request, u"Config saved")
             return redirect(request.path)
         context = template.RequestContext(request, {"forms": forms})
         return render_to_response("admin/dbconfig_list.html", context)
 
 
-if models.registry:
+if utils.registry:
     admin.site.register(models.DbConfigValue, ConfigAdmin)
